#if !defined(_ADATA_ADB3_IOCTL_ADB3_H)
#define _ADATA_ADB3_IOCTL_ADB3_H

/*
** File: ioctl_adb3.h  
** Project: ADB3 API library
** Purpose: Defines the IOCTL codes and structures for the ADB3 API
**
** (C) Copyright Alpha Data 2011-2015
**
** This file expects the following definitions to exist in the namespace:
**
**   ADB3_NEED_THUNK               - if building for a platform where thunking to 32 bits is required (e.g. x86_64)
*/

#define ADB3_IOCTLCODE_GETCOOPLEVEL                 (1)
#define ADB3_IOCTLCODE_SETCOOPLEVEL                 (2)
#define ADB3_IOCTLCODE_GETPRODTESTINFO              (3)
#define ADB3_IOCTLCODE_SERVICEIRQ                   (4)
#define ADB3_IOCTLCODE_GETUNIQUEID                  (5)
#define ADB3_IOCTLCODE_ABORTIPROG                   (6)
#define ADB3_IOCTLCODE_SCHEDULEIPROG                (7)
#define ADB3_IOCTLCODE_STATUSIPROG                  (8)
#define ADB3_IOCTLCODE_COMMANDAVR2                  (9)
#define ADB3_IOCTLCODE_GETAVR2STATUS                (10)
#define ADB3_IOCTLCODE_COMMANDAVR                   (11)
#define ADB3_IOCTLCODE_GETAVRSTATUS                 (12)

typedef union _IOCTLS_ADB3_GETCOOPLEVEL {
  struct {
    uint32_t cooperativeLevel;
  } out;
} IOCTLS_ADB3_GETCOOPLEVEL;

typedef union _IOCTLS_ADB3_GETPRODTESTINFO {
  struct {
    uint32_t serialNumber;
    uint8_t cpldRevision;
    uint8_t pcbRevision;
    uint8_t bridgeRevision;
    uint32_t bridgeDate;
    uint32_t bridgeTime;
  } out;
} IOCTLS_ADB3_GETPRODTESTINFO;

typedef union _IOCTLS_ADB3_GETUNIQUEID {
  struct {
    uint32_t serialNumber;
    uint32_t model;
  } out;
} IOCTLS_ADB3_GETUNIQUEID;

typedef union _IOCTLS_ADB3_SERVICEIRQ {
  struct {
    uint32_t bServicedIrq;
  } out;
} IOCTLS_ADB3_SERVICEIRQ;

typedef union _IOCTLS_ADB3_SETCOOPLEVEL {
  struct {
    uint32_t cooperativeLevel;
  } in;
} IOCTLS_ADB3_SETCOOPLEVEL;

typedef union _IOCTLS_ADB3_ABORTIPROG {
  struct {
    uint32_t targetIndex;
  } in;
} IOCTLS_ADB3_ABORTIPROG;

typedef union _IOCTLS_ADB3_SCHEDULEIPROG {
  struct {
    uint64_t address;
    uint32_t targetIndex;
    uint32_t flags;
    uint32_t delayMs;
  } in;
} IOCTLS_ADB3_SCHEDULEIPROG;

typedef union _IOCTLS_ADB3_STATUSIPROG {
  struct {
    uint32_t targetIndex;
  } in;
  struct {
    uint64_t address;
    uint32_t flags;
    uint32_t delayMs;
  } out;
} IOCTLS_ADB3_STATUSIPROG;

typedef union _IOCTLS_ADB3_COMMANDAVR2 {
  struct {
    uint32_t flags;
    uint32_t timeoutUs;
    uint32_t commandLength;
    const void* pCommand;
    uint32_t responseLength;
    void* pResponse;
  } in;
  struct {
    uint32_t actualResponseLength;
  } out;
} IOCTLS_ADB3_COMMANDAVR2;

#if defined(ADB3_NEED_THUNK)
typedef union _IOCTLS32_ADB3_COMMANDAVR2 {
  struct {
    uint32_t flags;
    uint32_t timeoutUs;
    uint32_t commandLength;
    const_ptr32_t pCommand;
    uint32_t responseLength;
    ptr32_t pResponse;
  } in;
  struct {
    uint32_t actualResponseLength;
  } out;
} IOCTLS32_ADB3_COMMANDAVR2;
#endif

typedef union _IOCTLS_ADB3_GETAVR2STATUS {
  union {
    struct {
      uint32_t serviceMode     : 1;
      uint32_t rxProtocolError : 1;
      uint32_t txFramingError  : 1;
      uint32_t reserved1       : 29;
    } flags;
    uint32_t asUint32;
  } out;
} IOCTLS_ADB3_GETAVR2STATUS;

typedef union _IOCTLS_ADB3_COMMANDAVR {
  struct {
    uint32_t flags;
    uint32_t timeoutUs;
    uint32_t commandLength;
    const void* pCommand;
    uint32_t responseLength;
    void* pResponse;
  } in;
  struct {
    uint32_t actualResponseLength;
  } out;
} IOCTLS_ADB3_COMMANDAVR;

#if defined(ADB3_NEED_THUNK)
typedef union _IOCTLS32_ADB3_COMMANDAVR {
  struct {
    uint32_t flags;
    uint32_t timeoutUs;
    uint32_t commandLength;
    const_ptr32_t pCommand;
    uint32_t responseLength;
    ptr32_t pResponse;
  } in;
  struct {
    uint32_t actualResponseLength;
  } out;
} IOCTLS32_ADB3_COMMANDAVR;
#endif

typedef union _IOCTLS_ADB3_GETAVRSTATUS {
  union {
    struct {
      uint32_t serviceMode     : 1;
      uint32_t rxProtocolError : 1;
      uint32_t txFramingError  : 1;
      uint32_t reserved1       : 29;
    } flags;
    uint32_t asUint32;
  } out;
} IOCTLS_ADB3_GETAVRSTATUS;

#endif
