/*
** File: combuf.c  
** Project: ADMXRC3 API module library
** Purpose: OS-independent code for ADMXRC3 API module library
**
** (C) Copyright Alpha Data 2011
**
** Implements the functions relating to common buffers for DMA.
*/

#include <admxrc3/combuf.h>
#include <admxrc3_platform.h>
#include "ioctl_admxrc3.h"

ADMXRC3_EXPORT ADMXRC3_STATUS
ADMXRC3_CALLING_CONVENTION
ADMXRC3_GetCommonBuffer(
  ADMXRC3_HANDLE hCard,
  unsigned int index,
  size_t* pSizeInBytes,
  uint64_t* pBusAddress)
{
  IOCTLS_ADMXRC3_GETCOMMONBUFFERINFO ioctl;
  ADMXRC3_STATUS status;

  ioctl.in.index = index;
  status = admxrc3Ioctl(hCard, ADMXRC3_IOCTLCODE_GETCOMMONBUFFERINFO, &ioctl, sizeof(ioctl.in), sizeof(ioctl.out));
  if (ADMXRC3_SUCCESS == status) {
    if (NULL != pSizeInBytes) {
      *pSizeInBytes = ioctl.out.size;
    }
    if (NULL != pBusAddress) {
      *pBusAddress = ioctl.out.busAddress;
    }
  }
  return status;
}

ADMXRC3_EXPORT ADMXRC3_STATUS
ADMXRC3_CALLING_CONVENTION
ADMXRC3_GetCommonBufferCount(
  ADMXRC3_HANDLE hCard,
  unsigned int* pCount)
{
  IOCTLS_ADMXRC3_GETCOMMONBUFFERCOUNT ioctl;
  ADMXRC3_STATUS status;

  if (pCount == NULL) {
    return ADMXRC3_NULL_POINTER;
  }

  status = admxrc3Ioctl(hCard, ADMXRC3_IOCTLCODE_GETCOMMONBUFFERCOUNT, &ioctl, 0, sizeof(ioctl.out));
  if (ADMXRC3_SUCCESS == status) {
    *pCount = (unsigned int)ioctl.out.count;
  }
  return status;
}

ADMXRC3_EXPORT ADMXRC3_STATUS
ADMXRC3_CALLING_CONVENTION
ADMXRC3_MapCommonBuffer(
  ADMXRC3_HANDLE hCard,
  uint64_t busAddress,
  size_t offsetInBytes,
  size_t sizeInBytes,
  void** ppVirtualAddress)
{
  ADMXRC3_STATUS status;
  unsigned int index, regionTag;
  uint64_t tmpAddress;

  if (ppVirtualAddress == NULL) {
    return ADMXRC3_NULL_POINTER;
  }

  for (index = 0;; index++) {
    status = ADMXRC3_GetCommonBuffer(hCard, index, NULL, &tmpAddress);
    if (ADMXRC3_SUCCESS == status) {
      if (busAddress == tmpAddress) {
        /* Found the buffer */
        regionTag = index + ADMXRC3_REGION_TAG_COMMONBUFFER;
        /* This has to be done in an OS-specific way */
        return admxrc3MapRegion(hCard, regionTag, offsetInBytes, sizeInBytes, ppVirtualAddress);
      }
    } else {
      switch (status) {
      case ADMXRC3_INVALID_HANDLE:
        return status;

      case ADMXRC3_INVALID_INDEX:
        return ADMXRC3_INVALID_REGION;

      default:
        return ADMXRC3_UNEXPECTED_ERROR;
      }
    }
  }
}

ADMXRC3_EXPORT ADMXRC3_STATUS
ADMXRC3_CALLING_CONVENTION
ADMXRC3_UnmapCommonBuffer(
  ADMXRC3_HANDLE hCard,
  void* pVirtualAddress)
{
  if (NULL == pVirtualAddress) {
    return ADMXRC3_NULL_POINTER;
  }

  /* This has to be done in an OS-specific way */
  return admxrc3UnmapRegion(hCard, pVirtualAddress);
}
