#ifndef _ADATA_PLATFORM_EVENT_H
#define _ADATA_PLATFORM_EVENT_H

#include <vxWorks.h>
#include <semLib.h>
#include <sysLib.h>

#define PORTABLE_EVENT_HANDLE_INVALID_VALUE NULL
typedef SEM_ID PortableEventHandle;

typedef enum _PortableEventStatus {
  PortableEventOk = 0,
  PortableEventTimeout = 1,
  PortableEventFailed = 2
} PortableEventStatus;

#if defined(_WRS_INLINE)
_WRS_INLINE PortableEventHandle
#else
static __inline__ PortableEventHandle
#endif
portableEventCreate(
  void)
{
  return semBCreate(0, SEM_EMPTY);
}

#if defined(_WRS_INLINE)
_WRS_INLINE void
#else
static __inline__ void
#endif
portableEventDestroy(
  PortableEventHandle e)
{
  semDelete(e);
}

#if defined(_WRS_INLINE)
_WRS_INLINE void
#else
static __inline__ void
#endif
portableEventSignal(
  PortableEventHandle e)
{
  semGive(e);
}

#if defined(_WRS_INLINE)
_WRS_INLINE PortableEventStatus
#else
static __inline__ PortableEventStatus
#endif
portableEventWait(
  PortableEventHandle e)
{
  if (OK == semTake(e, WAIT_FOREVER)) {
    return PortableEventOk;
  } else {
    if (errno == S_objLib_OBJ_TIMEOUT) {
      return PortableEventTimeout;
    } else {
      return PortableEventFailed;
    }
  }
}

#if defined(_WRS_INLINE)
_WRS_INLINE PortableEventStatus
#else
static __inline__ PortableEventStatus
#endif
portableEventWaitTimeout(
  PortableEventHandle e,
  unsigned int timeoutMs)
{
  const int k = 1000;
  INT64 ticks;

  ticks = ((INT64)timeoutMs * sysClkRateGet() + k - 1) / k;
  if (OK == semTake(e, WAIT_FOREVER)) {
    return PortableEventOk;
  } else {
    if (errno == S_objLib_OBJ_TIMEOUT) {
      return PortableEventTimeout;
    } else {
      return PortableEventFailed;
    }
  }
}

#endif
